<?php
/**
 * Blackbird ContentManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_ContentManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 * @license		http://www.advancedcontentmanager.com/license/
 */
namespace Blackbird\ContentManager\Model;

use Blackbird\ContentManager\Api\Data\ContentType\Layout\ItemInterface as Item;

class ContentList extends \Blackbird\ContentManager\Model\AbstractModel 
    implements \Blackbird\ContentManager\Api\Data\ContentListInterface,
               \Blackbird\ContentManager\Api\ContentLayoutInterface,
               \Magento\Framework\DataObject\IdentityInterface
{
    /**
     * Content manager content cache tag
     */
    const CACHE_TAG = 'contentmanager_contentlist';
    
    const ENTITY_TYPE = 'contenttype_contentlist';
    
    /**
     * Available layout item
     * (even put group in first position)
     *
     * @var array
     */
    protected $_availableLayoutItem = [
        'group', 'field', 'block',
    ];
    
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Group\CollectionFactory
     */
    protected $_groupItemCollectionFactory;
    
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Field\CollectionFactory
     */
    protected $_fieldItemCollectionFactory;
    
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Block\CollectionFactory
     */
    protected $_blockItemCollectionFactory;
    
    /**
     * @var \Blackbird\ContentManager\Model\Factory
     */
    protected $_modelFactory;
    
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Blackbird\ContentManager\Model\Rule
     */
    public $rule;
    
    /**
     * @var \Blackbird\ContentManager\Helper\UrlRewriteGenerator
     */
    protected $_urlRewriteHelper;
    
    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Blackbird\ContentManager\Model\Factory $modelFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Group\CollectionFactory $groupItemCollectionFactory
     * @param \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Field\CollectionFactory $fieldItemCollectionFactory
     * @param \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Block\CollectionFactory $blockItemCollectionFactory
     * @param \Blackbird\ContentManager\Model\Rule $rule
     * @param \Blackbird\ContentManager\Helper\UrlRewriteGenerator $urlRewriteHelper
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Blackbird\ContentManager\Model\Factory $modelFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Group\CollectionFactory $groupItemCollectionFactory,
        \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Field\CollectionFactory $fieldItemCollectionFactory,
        \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Block\CollectionFactory $blockItemCollectionFactory,
        \Blackbird\ContentManager\Model\Rule $rule,
        \Blackbird\ContentManager\Helper\UrlRewriteGenerator $urlRewriteHelper,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = array()
    ) {
        $this->_modelFactory = $modelFactory;
        $this->_storeManager = $storeManager;
        $this->_groupItemCollectionFactory = $groupItemCollectionFactory;
        $this->_fieldItemCollectionFactory = $fieldItemCollectionFactory;
        $this->_blockItemCollectionFactory = $blockItemCollectionFactory;
        $this->rule = $rule;
        $this->_urlRewriteHelper = $urlRewriteHelper;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }
    
    /**
     * @return void
     */
    public function _construct()
    {
        parent::_construct();
        $this->_init('Blackbird\ContentManager\Model\ResourceModel\ContentList');
        $this->setIdFieldName(static::ID);
    }

    /**
     * Get identities
     *
     * @return array
     */
    public function getIdentities()
    {
        return [
            self::CACHE_TAG . '_' . $this->getId(),
            ContentType::CACHE_TAG . '_' . $this->getContentType()->getId()
        ];
    }
    
    /**
     * Get the content type
     * 
     * @return ContentType
     */
    public function getContentType()
    {
        if (!$this->hasData('content_type')) {
            $contentType = $this->_modelFactory->create(ContentType::class)->load($this->getCtId());
            $this->setData('content_type', $contentType);
        }
        
        return $this->getData('content_type');
    }
    
    /**
     * Return the stores
     * 
     * @return Store[]
     */
    public function getStores()
    {
        if (!$this->hasData('stores')) {
            $stores = $this->getResource()->lookupStoreIds($this->getId());
            if (empty($stores)) {
                $stores = [$this->_storeManager->getStore()->getId()];
            }
            $this->setData('stores', $stores);
        }
        
        return $this->getData('stores');
    }
    
    /**
     * Return list of layout items values
     * 
     * @return array
     */
    public function getLayoutItemCollection()
    {
        $items = array();
        
        // Group items
        $collection = $this->getLayoutGroupItemCollection();
        foreach ($collection as $group) {
            $items[$group->getSortOrder()] = $group;
        }
        
        // Field items
        $collection = $this->getLayoutFieldItemCollection();
        foreach ($collection as $field) {
            $items[$field->getSortOrder()] = $field;
        }
        
        // Block items
        $collection = $this->getLayoutBlockItemCollection();
        foreach ($collection as $block) {
            $items[$block->getSortOrder()] = $block;
        }
        
        ksort($items);
        
        return $items;
    }
    
    /**
     * Retrieve collection of layout group item
     * 
     * @return \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Group\Collection
     */
    public function getLayoutGroupItemCollection()
    {
        $collection = $this->_groupItemCollectionFactory->create();
        
        $collection->addFieldToFilter(static::ID, $this->getId())
                   ->setOrder(Item::SORT_ORDER);
        
        return $collection;
    }
    
    /**
     * Retrieve collection of layout field item
     * 
     * @return \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Field\Collection
     */
    public function getLayoutFieldItemCollection()
    {
        $collection = $this->_fieldItemCollectionFactory->create();
        
        $collection->addFieldToFilter(static::ID, $this->getId())
                   ->setOrder(Item::SORT_ORDER);
        
        return $collection;
    }
    
    /**
     * Retrieve collection of layout block item
     * 
     * @return \Blackbird\ContentManager\Model\ResourceModel\ContentList\Layout\Block\Collection
     */
    public function getLayoutBlockItemCollection()
    {
        $collection = $this->_blockItemCollectionFactory->create();
        
        $collection->addFieldToFilter(static::ID, $this->getId())
                   ->setOrder(Item::SORT_ORDER);
        
        return $collection;
    }
    
    /**
     * Retrieve the url key (UrlKey as Request Path)
     * 
     * @return string
     */
    public function getUrl()
    {
        return $this->getUrlKey();
    }
    
    /**
     * Processing object after save data
     *
     * @return $this
     */
    public function afterSave()
    {
        // Rewrite url generation
        $this->generateUrls();
        
        // Save the layout items
        $this->saveLayoutItems();
        
        return parent::afterSave();
    }
    
    /**
     * Generate the url rewrite
     * 
     * @return void
     */
    public function generateUrls()
    {
        $urls = [];
        $storeIds = [];
        
        if (($key = array_search(\Magento\Store\Model\Store::DEFAULT_STORE_ID, $this->getStores())) !== false) {
            foreach ($this->_storeManager->getStores() as $store) {
                $storeIds[] = $store->getId();
            }
            $storeIds = array_unique($storeIds);
        }
        
        foreach ($storeIds as $store) {
            $urls[] = [
                'entity_type' => static::ENTITY_TYPE,
                'entity_id' => $this->getId(),
                'request_path' => $this->getUrlKey(),
                'target_path' => 'contentmanager/index/contentlist/contentlist_id/' . $this->getId(),
                'store_id' => $store
            ];
        }
        
        $this->_urlRewriteHelper->deleteUrlRewrite(static::ENTITY_TYPE, $this->getId());
        $this->_urlRewriteHelper->addUrlRewrites($urls);
    }

    /**
     * Save the custom layout configuration
     * 
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return $this
     */
    protected function saveLayoutItems()
    {
        // Parent id associated to their real Id
        $groupItem = array();
        // Retrieve layout items
        $data = [];
        
        if ($this->hasData('after_save_item')) {
            $data = $this->getData('after_save_item');
        }
        
        // Block, field and group items
        foreach ($this->_availableLayoutItem as $type) {
            // Retrieve items by type
            $items = (isset($data[$type]) && is_array($data[$type])) ? $data[$type] : [];
            
            // Items
            foreach ($items as $key => $item) {
                $itemModel = $this->_modelFactory->create('\Blackbird\ContentManager\Model\ContentList\Layout\\' . ucfirst(strtolower($type)));
                
                if (!empty($item['id'])) {
                    $itemModel->load($item['id']);
                    
                    if (!empty($itemModel->getId())) {
                        if (!empty($item['is_delete'])) {
                            $itemModel->delete();
                            continue;
                        }
                    } else {
                        // The item no longer exists
                        continue;
                    }
                }
                // If the item is not saved and has been deleted
                if (!empty($item['is_delete'])) {
                    continue;
                }
                
                // Set data to the item
                $this->setItemData($itemModel, $item, $type);
                
                try {
                    $itemModel->save();
                    
                    if ($type === 'group') {
                        $groupItem[$key] = $itemModel->getId();
                    }
                } catch (\Exception $e) {
                    throw new \Magento\Framework\Exception\LocalizedException(
                        __('Something went wrong while saving the layout item.')
                    );
                }
            }
        }
        
        return $this;
    }
    
    /**
     * Set data to an item
     * 
     * @param Item $itemModel
     * @param array $item
     * @param string $type
     * @return Item
     */
    protected function setItemData(Item $itemModel, array $item, $type)
    {
        // Set data to the item
        $itemModel->setData($item);
        $itemModel->setData(static::ID, $this->getId());
        // Set id of the item if it exists
        if (!empty($item['id'])) {
            $itemModel->setData($itemModel::ID, $item['id']);
        }

        // Manage format for item type of field
        if ($type === 'field') {
            $itemModel->setFormat($this->getFormatSerialize($item));
            $cfId = (!empty($item['custom_field_id'])) ? $item['custom_field_id'] : null;
            $itemModel->setCustomFieldId($cfId);
        }

        // Set parent layout group item
        if (!empty($item['parent_id'])) {
            $itemModel->setData($itemModel::PARENT_ID, ['parent_id']);
        }
        
        return $itemModel;
    }
    
    /**
     * Retrieves the format for an item and serialize it
     * 
     * @param array $item
     * @return string
     */
    protected function getFormatSerialize(array $item)
    {
        $result = serialize([
            'type' => isset($item['format']) ? $item['format'] : '',
            'extra' => isset($item['format_extra']) ? $item['format_extra'] : '',
            'height' => isset($item['format_height']) ? $item['format_height'] : '',
            'width' => isset($item['format_width']) ? $item['format_width'] : '',
            'link' => isset($item['link']) ? $item['link'] : '',
        ]);
        
        return $result;
    }
    
    /**
     * Processing object before delete data
     *
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function beforeDelete()
    {
        // Delete UrlRewrite
        $this->_urlRewriteHelper->deleteUrlRewrite(static::ENTITY_TYPE, $this->getId());
        
        // Delete Layout Items
        $this->deleteLayoutItems();
        
        return parent::beforeDelete();
    }
    
    /**
     * Delete the layout items
     * 
     * @return $this
     */
    protected function deleteLayoutItems()
    {
        $items = $this->getLayoutItemCollection();
        krsort($items);
        
        foreach ($items as $item) {
            $item->delete();
        }
        
        return $this;
    }
}
